@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 from functools import wraps
 
 import numpy as np
@@ -206,6 +204,7 @@ def getmaskarray(a):
 
 
 def _masked_array(data, mask=np.ma.nomask, **kwargs):
+    kwargs.pop("chunks", None)  # A Dask kwarg, not NumPy.
     dtype = kwargs.pop("masked_dtype", None)
     return np.ma.masked_array(data, mask=mask, dtype=dtype, **kwargs)
 
