@@ -58,6 +58,8 @@ def meta_from_array(x, ndim=None, dtype=None):
 
     if x is None:
         x = np.ndarray
+    elif dtype is None and hasattr(x, "dtype"):
+        dtype = x.dtype
 
     if isinstance(x, type):
         x = x(shape=(0,) * (ndim or 0), dtype=dtype)
@@ -101,7 +103,22 @@ def meta_from_array(x, ndim=None, dtype=None):
         meta = np.array(meta)
 
     if dtype and meta.dtype != dtype:
-        meta = meta.astype(dtype)
+        try:
+            meta = meta.astype(dtype)
+        except ValueError as e:
+            if (
+                any(
+                    s in str(e)
+                    for s in [
+                        "invalid literal",
+                        "could not convert string to float",
+                    ]
+                )
+                and meta.dtype.kind in "SU"
+            ):
+                meta = np.array([]).astype(dtype)
+            else:
+                raise e
 
     return meta
 
@@ -126,14 +143,23 @@ def compute_meta(func, _dtype, *args, **kwargs):
                     kwargs_meta["computing_meta"] = True
                 meta = func(*args_meta, **kwargs_meta)
             except TypeError as e:
-                if (
-                    "unexpected keyword argument" in str(e)
-                    or "is an invalid keyword for" in str(e)
-                    or "Did not understand the following kwargs" in str(e)
+                if any(
+                    s in str(e)
+                    for s in [
+                        "unexpected keyword argument",
+                        "is an invalid keyword for",
+                        "Did not understand the following kwargs",
+                    ]
                 ):
                     raise
                 else:
                     return None
+            except ValueError as e:
+                # min/max functions have no identity, attempt to use the first meta
+                if "zero-size array to reduction operation" in str(e):
+                    meta = args_meta[0]
+                else:
+                    return None
             except Exception:
                 return None
 
@@ -244,23 +270,50 @@ def assert_eq(a, b, check_shape=True, check_graph=True, check_meta=True, **kwarg
             )
 
     try:
-        assert a.shape == b.shape
+        assert (
+            a.shape == b.shape
+        ), f"a and b have different shapes (a: {a.shape}, b: {b.shape})"
         if check_meta:
             if hasattr(a, "_meta") and hasattr(b, "_meta"):
                 assert_eq(a._meta, b._meta)
             if hasattr(a_original, "_meta"):
-                assert a_original._meta.ndim == a.ndim
+                msg = (
+                    f"compute()-ing 'a' changes its number of dimensions "
+                    f"(before: {a_original._meta.ndim}, after: {a.ndim})"
+                )
+                assert a_original._meta.ndim == a.ndim, msg
                 if a_meta is not None:
-                    assert type(a_original._meta) == type(a_meta)
+                    msg = (
+                        f"compute()-ing 'a' changes its type "
+                        f"(before: {type(a_original._meta)}, after: {type(a_meta)})"
+                    )
+                    assert type(a_original._meta) == type(a_meta), msg
                     if not (np.isscalar(a_meta) or np.isscalar(a_computed)):
-                        assert type(a_meta) == type(a_computed)
+                        msg = (
+                            f"compute()-ing 'a' results in a different type than implied by its metadata "
+                            f"(meta: {type(a_meta)}, computed: {type(a_computed)})"
+                        )
+                        assert type(a_meta) == type(a_computed), msg
             if hasattr(b_original, "_meta"):
-                assert b_original._meta.ndim == b.ndim
+                msg = (
+                    f"compute()-ing 'b' changes its number of dimensions "
+                    f"(before: {b_original._meta.ndim}, after: {b.ndim})"
+                )
+                assert b_original._meta.ndim == b.ndim, msg
                 if b_meta is not None:
-                    assert type(b_original._meta) == type(b_meta)
+                    msg = (
+                        f"compute()-ing 'b' changes its type "
+                        f"(before: {type(b_original._meta)}, after: {type(b_meta)})"
+                    )
+                    assert type(b_original._meta) == type(b_meta), msg
                     if not (np.isscalar(b_meta) or np.isscalar(b_computed)):
-                        assert type(b_meta) == type(b_computed)
-        assert allclose(a, b, **kwargs)
+                        msg = (
+                            f"compute()-ing 'b' results in a different type than implied by its metadata "
+                            f"(meta: {type(b_meta)}, computed: {type(b_computed)})"
+                        )
+                        assert type(b_meta) == type(b_computed), msg
+        msg = "found values in 'a' and 'b' which differ by more than the allowed amount"
+        assert allclose(a, b, **kwargs), msg
         return True
     except TypeError:
         pass
@@ -386,7 +439,7 @@ def svd_flip(u, v, u_based_decision=False):
     else:
         dtype = v.dtype
         signs = np.sum(v, axis=1, keepdims=True).T
-    signs = dtype.type(2) * ((signs >= 0) - dtype.type(0.5))
+    signs = 2.0 * ((signs >= 0) - 0.5).astype(dtype)
     # Force all singular vectors into same half-space
     u, v = u * signs, v * signs.T
     return u, v
