@@ -24,6 +24,8 @@ from .utils import (
     raise_on_meta_error,
     is_series_like,
     is_dataframe_like,
+    PANDAS_GT_100,
+    PANDAS_GT_110,
 )
 from ..base import tokenize
 from ..utils import derived_from, M, funcname, itemgetter
@@ -161,24 +163,26 @@ def _groupby_raise_unaligned(df, **kwargs):
 
 
 def _groupby_slice_apply(
-    df, grouper, key, func, *args, group_keys=True, dropna=None, **kwargs
+    df, grouper, key, func, *args, group_keys=True, dropna=None, observed=None, **kwargs
 ):
     # No need to use raise if unaligned here - this is only called after
     # shuffling, which makes everything aligned already
     dropna = {"dropna": dropna} if dropna is not None else {}
-    g = df.groupby(grouper, group_keys=group_keys, **dropna)
+    observed = {"observed": observed} if observed is not None else {}
+    g = df.groupby(grouper, group_keys=group_keys, **observed, **dropna)
     if key:
         g = g[key]
     return g.apply(func, *args, **kwargs)
 
 
 def _groupby_slice_transform(
-    df, grouper, key, func, *args, group_keys=True, dropna=None, **kwargs
+    df, grouper, key, func, *args, group_keys=True, dropna=None, observed=None, **kwargs
 ):
     # No need to use raise if unaligned here - this is only called after
     # shuffling, which makes everything aligned already
     dropna = {"dropna": dropna} if dropna is not None else {}
-    g = df.groupby(grouper, group_keys=group_keys, **dropna)
+    observed = {"observed": observed} if observed is not None else {}
+    g = df.groupby(grouper, group_keys=group_keys, **observed, **dropna)
     if key:
         g = g[key]
 
@@ -273,18 +277,24 @@ class Aggregation(object):
 
 
 def _groupby_aggregate(
-    df, aggfunc=None, levels=None, dropna=None, sort=False, **kwargs
+    df, aggfunc=None, levels=None, dropna=None, sort=False, observed=None, **kwargs
 ):
     dropna = {"dropna": dropna} if dropna is not None else {}
-    return aggfunc(df.groupby(level=levels, sort=sort, **dropna), **kwargs)
+    if not PANDAS_GT_100 and observed:
+        raise NotImplementedError("``observed`` is only supported for pandas >= 1.0.0")
+    observed = {"observed": observed} if observed is not None else {}
+
+    grouped = df.groupby(level=levels, sort=sort, **observed, **dropna)
+    return aggfunc(grouped, **kwargs)
 
 
-def _apply_chunk(df, *index, dropna=None, **kwargs):
+def _apply_chunk(df, *index, dropna=None, observed=None, **kwargs):
     func = kwargs.pop("chunk")
     columns = kwargs.pop("columns")
     dropna = {"dropna": dropna} if dropna is not None else {}
-    g = _groupby_raise_unaligned(df, by=index, **dropna)
+    observed = {"observed": observed} if observed is not None else {}
 
+    g = _groupby_raise_unaligned(df, by=index, **observed, **dropna)
     if is_series_like(df) or columns is None:
         return func(g, **kwargs)
     else:
@@ -400,7 +410,7 @@ def _cov_chunk(df, *index):
     Returns
     -------
     tuple
-        Processed X, Multipled Cols,
+        Processed X, Multiplied Cols,
     """
     if is_series_like(df):
         df = df.to_frame()
@@ -936,14 +946,16 @@ def _compute_sum_of_squares(grouped, column):
     return df.groupby(keys).sum()
 
 
-def _agg_finalize(df, aggregate_funcs, finalize_funcs, level, sort=False):
+def _agg_finalize(df, aggregate_funcs, finalize_funcs, level, sort=False, **kwargs):
     # finish the final aggregation level
-    df = _groupby_apply_funcs(df, funcs=aggregate_funcs, level=level, sort=sort)
+    df = _groupby_apply_funcs(
+        df, funcs=aggregate_funcs, level=level, sort=sort, **kwargs
+    )
 
     # and finalize the result
     result = collections.OrderedDict()
-    for result_column, func, kwargs in finalize_funcs:
-        result[result_column] = func(df, **kwargs)
+    for result_column, func, finalize_kwargs in finalize_funcs:
+        result[result_column] = func(df, **finalize_kwargs)
 
     return type(df)(result)
 
@@ -1029,10 +1041,21 @@ class _GroupBy(object):
     sort: bool, defult None
         Passed along to aggregation methods. If allowed,
         the output aggregation will have sorted keys.
+    observed: bool, default False
+        This only applies if any of the groupers are Categoricals.
+        If True: only show observed values for categorical groupers.
+        If False: show all values for categorical groupers.
     """
 
     def __init__(
-        self, df, by=None, slice=None, group_keys=True, dropna=None, sort=None
+        self,
+        df,
+        by=None,
+        slice=None,
+        group_keys=True,
+        dropna=None,
+        sort=None,
+        observed=None,
     ):
 
         assert isinstance(df, (DataFrame, Series))
@@ -1076,8 +1099,13 @@ class _GroupBy(object):
         if dropna is not None:
             self.dropna["dropna"] = dropna
 
+        # Hold off on setting observed by default: https://github.com/dask/dask/issues/6951
+        self.observed = {}
+        if observed is not None:
+            self.observed["observed"] = observed
+
         self._meta = self.obj._meta.groupby(
-            index_meta, group_keys=group_keys, **self.dropna
+            index_meta, group_keys=group_keys, **self.observed, **self.dropna
         )
 
     @property
@@ -1099,7 +1127,12 @@ class _GroupBy(object):
         else:
             index_meta = self.index
 
-        grouped = sample.groupby(index_meta, group_keys=self.group_keys, **self.dropna)
+        grouped = sample.groupby(
+            index_meta,
+            group_keys=self.group_keys,
+            **self.observed,
+            **self.dropna,
+        )
         return _maybe_slice(grouped, self._slice)
 
     def _aca_agg(
@@ -1127,14 +1160,22 @@ class _GroupBy(object):
             else [self.obj] + self.index,
             chunk=_apply_chunk,
             chunk_kwargs=dict(
-                chunk=func, columns=columns, **chunk_kwargs, **self.dropna
+                chunk=func,
+                columns=columns,
+                **self.observed,
+                **chunk_kwargs,
+                **self.dropna,
             ),
             aggregate=_groupby_aggregate,
             meta=meta,
             token=token,
             split_every=split_every,
             aggregate_kwargs=dict(
-                aggfunc=aggfunc, levels=levels, **aggregate_kwargs, **self.dropna
+                aggfunc=aggfunc,
+                levels=levels,
+                **self.observed,
+                **aggregate_kwargs,
+                **self.dropna,
             ),
             split_out=split_out,
             split_out_setup=split_out_on_index,
@@ -1161,7 +1202,7 @@ class _GroupBy(object):
             columns=columns,
             token=name_part,
             meta=meta,
-            **self.dropna
+            **self.dropna,
         )
 
         cumpart_raw_frame = (
@@ -1190,7 +1231,7 @@ class _GroupBy(object):
             chunk=M.last,
             meta=meta,
             token=name_last,
-            **self.dropna
+            **self.dropna,
         )
 
         # aggregate cumulated partitions and its previous last element
@@ -1554,15 +1595,27 @@ class _GroupBy(object):
         else:
             chunk_args = [self.obj] + self.index
 
+        if not PANDAS_GT_110 and self.dropna:
+            raise NotImplementedError(
+                "dropna is not a valid argument for dask.groupby.agg"
+                f"if pandas < 1.1.0. Pandas version is {pd.__version__}"
+            )
+
         return aca(
             chunk_args,
             chunk=_groupby_apply_funcs,
-            chunk_kwargs=dict(funcs=chunk_funcs),
+            chunk_kwargs=dict(funcs=chunk_funcs, **self.observed, **self.dropna),
             combine=_groupby_apply_funcs,
-            combine_kwargs=dict(funcs=aggregate_funcs, level=levels),
+            combine_kwargs=dict(
+                funcs=aggregate_funcs, level=levels, **self.observed, **self.dropna
+            ),
             aggregate=_agg_finalize,
             aggregate_kwargs=dict(
-                aggregate_funcs=aggregate_funcs, finalize_funcs=finalizers, level=levels
+                aggregate_funcs=aggregate_funcs,
+                finalize_funcs=finalizers,
+                level=levels,
+                **self.observed,
+                **self.dropna,
             ),
             token="aggregate",
             split_every=split_every,
@@ -1653,8 +1706,9 @@ class _GroupBy(object):
             token=funcname(func),
             *args,
             group_keys=self.group_keys,
+            **self.observed,
             **self.dropna,
-            **kwargs
+            **kwargs,
         )
 
         return df3
@@ -1741,8 +1795,9 @@ class _GroupBy(object):
             token=funcname(func),
             *args,
             group_keys=self.group_keys,
+            **self.observed,
             **self.dropna,
-            **kwargs
+            **kwargs,
         )
 
         return df3
@@ -1797,8 +1852,10 @@ class SeriesGroupBy(_GroupBy):
 
     _token_prefix = "series-groupby-"
 
-    def __init__(self, df, by=None, slice=None, **kwargs):
+    def __init__(self, df, by=None, slice=None, observed=None, **kwargs):
         # for any non series object, raise pandas-compat error message
+        # Hold off on setting observed by default: https://github.com/dask/dask/issues/6951
+        observed = {"observed": observed} if observed is not None else {}
 
         if isinstance(df, Series):
             if isinstance(by, Series):
@@ -1809,12 +1866,13 @@ class SeriesGroupBy(_GroupBy):
 
                 non_series_items = [item for item in by if not isinstance(item, Series)]
                 # raise error from pandas, if applicable
-                df._meta.groupby(non_series_items)
+
+                df._meta.groupby(non_series_items, **observed)
             else:
                 # raise error from pandas, if applicable
-                df._meta.groupby(by)
+                df._meta.groupby(by, **observed)
 
-        super().__init__(df, by=by, slice=slice, **kwargs)
+        super().__init__(df, by=by, slice=slice, **observed, **kwargs)
 
     @derived_from(pd.core.groupby.SeriesGroupBy)
     def nunique(self, split_every=None, split_out=1):
@@ -1873,7 +1931,7 @@ class SeriesGroupBy(_GroupBy):
     def value_counts(self, split_every=None, split_out=1):
         return self._aca_agg(
             token="value_counts",
-            func=M.value_counts,
+            func=_value_counts,
             aggfunc=_value_counts_aggregate,
             split_every=split_every,
             split_out=split_out,
@@ -1898,6 +1956,13 @@ def _unique_aggregate(series_gb, name=None):
     return ret
 
 
+def _value_counts(x, **kwargs):
+    if len(x):
+        return M.value_counts(x, **kwargs)
+    else:
+        return pd.Series(dtype=int)
+
+
 def _value_counts_aggregate(series_gb):
     to_concat = {k: v.sum(level=1) for k, v in series_gb}
     names = list(series_gb.obj.index.names)
