@@ -11,7 +11,7 @@ There are two important cases:
 2.  We combine along an unpartitioned index or other column
 
 In the first case we know which partitions of each dataframe interact with
-which others.  This lets uss be significantly more clever and efficient.
+which others.  This lets us be significantly more clever and efficient.
 
 In the second case each partition from one dataset interacts with all
 partitions from the other.  We handle this through a shuffle operation.
@@ -59,7 +59,7 @@ import warnings
 from tlz import merge_sorted, unique, first
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_dtype_equal, is_categorical_dtype
+from pandas.api.types import is_dtype_equal, is_categorical_dtype, union_categoricals
 
 from ..base import tokenize, is_dask_collection
 from ..highlevelgraph import HighLevelGraph
@@ -91,7 +91,7 @@ from ..utils import M
 
 
 def align_partitions(*dfs):
-    """ Mutually partition and align DataFrame blocks
+    """Mutually partition and align DataFrame blocks
 
     This serves as precursor to multi-dataframe operations like join, concat,
     or merge.
@@ -168,7 +168,7 @@ def _maybe_align_partitions(args):
 
 
 def require(divisions, parts, required=None):
-    """ Clear out divisions where required components are not present
+    """Clear out divisions where required components are not present
 
     In left, right, or inner joins we exclude portions of the dataset if one
     side or the other is not present.  We can achieve this at the partition
@@ -231,7 +231,48 @@ allowed_right = ("inner", "right")
 
 def merge_chunk(lhs, *args, **kwargs):
     empty_index_dtype = kwargs.pop("empty_index_dtype", None)
-    out = lhs.merge(*args, **kwargs)
+    categorical_columns = kwargs.pop("categorical_columns", None)
+
+    rhs, *args = args
+    left_index = kwargs.get("left_index", False)
+    right_index = kwargs.get("right_index", False)
+
+    if categorical_columns is not None and PANDAS_GT_100:
+        for col in categorical_columns:
+            left = None
+            right = None
+
+            if col in lhs:
+                left = lhs[col]
+            elif col == kwargs.get("right_on", None) and left_index:
+                if is_categorical_dtype(lhs.index):
+                    left = lhs.index
+
+            if col in rhs:
+                right = rhs[col]
+            elif col == kwargs.get("left_on", None) and right_index:
+                if is_categorical_dtype(rhs.index):
+                    right = rhs.index
+
+            dtype = "category"
+            if left is not None and right is not None:
+                dtype = union_categoricals(
+                    [left.astype("category").values, right.astype("category").values]
+                ).dtype
+
+            if left is not None:
+                if isinstance(left, pd.Index):
+                    lhs.index = left.astype(dtype)
+                else:
+                    lhs = lhs.assign(**{col: left.astype(dtype)})
+            if right is not None:
+                if isinstance(right, pd.Index):
+                    rhs.index = right.astype(dtype)
+                else:
+                    rhs = rhs.assign(**{col: right.astype(dtype)})
+
+    out = lhs.merge(rhs, *args, **kwargs)
+
     # Workaround pandas bug where if the output result of a merge operation is
     # an empty dataframe, the output index is `int64` in all cases, regardless
     # of input dtypes.
@@ -253,6 +294,7 @@ def merge_indexed_dataframes(lhs, rhs, left_index=True, right_index=True, **kwar
 
     meta = lhs._meta_nonempty.merge(rhs._meta_nonempty, **kwargs)
     kwargs["empty_index_dtype"] = meta.index.dtype
+    kwargs["categorical_columns"] = meta.select_dtypes(include="category").columns
 
     dsk = dict()
     for i, (a, b) in enumerate(parts):
@@ -276,7 +318,7 @@ def hash_join(
     shuffle=None,
     indicator=False,
 ):
-    """ Join two DataFrames on particular columns with hash join
+    """Join two DataFrames on particular columns with hash join
 
     This shuffles both datasets on the joined column and then performs an
     embarrassingly parallel join partition-by-partition
@@ -323,6 +365,8 @@ def hash_join(
     name = "hash-join-" + token
 
     kwargs["empty_index_dtype"] = meta.index.dtype
+    kwargs["categorical_columns"] = meta.select_dtypes(include="category").columns
+
     dsk = {
         (name, i): (apply, merge_chunk, [(lhs2._name, i), (rhs2._name, i)], kwargs)
         for i in range(npartitions)
@@ -334,11 +378,13 @@ def hash_join(
 
 
 def single_partition_join(left, right, **kwargs):
-    # if the merge is perfomed on_index, divisions can be kept, otherwise the
-    # new index will not necessarily correspond the current divisions
+    # if the merge is performed on_index, divisions can be kept, otherwise the
+    # new index will not necessarily correspond with the current divisions
 
     meta = left._meta_nonempty.merge(right._meta_nonempty, **kwargs)
     kwargs["empty_index_dtype"] = meta.index.dtype
+    kwargs["categorical_columns"] = meta.select_dtypes(include="category").columns
+
     name = "merge-" + tokenize(left, right, **kwargs)
     if left.npartitions == 1 and kwargs["how"] in allowed_right:
         left_key = first(left.__dask_keys__())
@@ -375,8 +421,7 @@ def single_partition_join(left, right, **kwargs):
 
 
 def warn_dtype_mismatch(left, right, left_on, right_on):
-    """ Checks for merge column dtype mismatches and throws a warning (#4574)
-    """
+    """Checks for merge column dtype mismatches and throws a warning (#4574)"""
 
     if not isinstance(left_on, list):
         left_on = [left_on]
@@ -532,6 +577,8 @@ def merge(
             suffixes=suffixes,
             indicator=indicator,
         )
+        categorical_columns = meta.select_dtypes(include="category").columns
+
         if merge_indexed_left and left.known_divisions:
             right = rearrange_by_divisions(
                 right, right_on, left.divisions, max_branch, shuffle=shuffle
@@ -556,6 +603,7 @@ def merge(
             suffixes=suffixes,
             indicator=indicator,
             empty_index_dtype=meta.index.dtype,
+            categorical_columns=categorical_columns,
         )
     # Catch all hash join
     else:
@@ -591,7 +639,7 @@ def most_recent_tail_summary(left, right, by=None):
 
 
 def compute_tails(ddf, by=None):
-    """ For each partition, returns the last row of the most recent nonempty
+    """For each partition, returns the last row of the most recent nonempty
     partition.
     """
     empty = ddf._meta.iloc[0:0]
@@ -614,7 +662,7 @@ def most_recent_head_summary(left, right, by=None):
 
 
 def compute_heads(ddf, by=None):
-    """ For each partition, returns the first row of the next nonempty
+    """For each partition, returns the first row of the next nonempty
     partition.
     """
     empty = ddf._meta.iloc[0:0]
@@ -627,7 +675,7 @@ def compute_heads(ddf, by=None):
 
 
 def pair_partitions(L, R):
-    """ Returns which partitions to pair for the merge_asof algorithm and the
+    """Returns which partitions to pair for the merge_asof algorithm and the
     bounds on which to split them up
     """
     result = []
@@ -681,10 +729,9 @@ def merge_asof_padded(left, right, prev=None, next=None, **kwargs):
 
 def get_unsorted_columns(frames):
     """
-    Determine the unsorted colunn order.
+    Determine the unsorted column order.
 
     This should match the output of concat([frames], sort=False)
-    for pandas >=0.23
     """
     new_columns = pd.concat([frame._meta for frame in frames]).columns
     order = []
@@ -697,15 +744,6 @@ def get_unsorted_columns(frames):
     return order
 
 
-def concat_and_unsort(frames, columns):
-    """
-    Compatibility concat for Pandas <0.23.0
-
-    Concatenates and then selects the desired (unsorted) column order.
-    """
-    return pd.concat(frames)[columns]
-
-
 def _concat_compat(frames, left, right):
     if PANDAS_GT_100:
         # join_axes removed
@@ -976,7 +1014,7 @@ def concat(
     ignore_unknown_divisions=False,
     **kwargs
 ):
-    """ Concatenate DataFrames along rows.
+    """Concatenate DataFrames along rows.
 
     - When axis=0 (default), concatenate DataFrames row-wise:
 
@@ -1093,7 +1131,7 @@ def concat(
             if not ignore_unknown_divisions:
                 warnings.warn(
                     "Concatenating dataframes with unknown divisions.\n"
-                    "We're assuming that the indexes of each dataframes"
+                    "We're assuming that the indices of each dataframes"
                     " are \n aligned. This assumption is not generally "
                     "safe."
                 )
