@@ -4,9 +4,10 @@ import pandas as pd
 import pytest
 import numpy as np
 
-import dask.array as da
 import dask.dataframe as dd
-from dask.dataframe.utils import assert_eq, PANDAS_VERSION
+from dask.dataframe.utils import assert_eq
+from dask.dataframe._compat import PANDAS_GT_130
+
 
 N = 40
 df = pd.DataFrame(
@@ -21,8 +22,9 @@ df = pd.DataFrame(
 ddf = dd.from_pandas(df, 3)
 
 idx = (
-    pd.date_range("2016-01-01", freq="3s", periods=100)
-    | pd.date_range("2016-01-01", freq="5s", periods=100)
+    pd.date_range("2016-01-01", freq="3s", periods=100).union(
+        pd.date_range("2016-01-01", freq="5s", periods=100)
+    )
 )[:N]
 
 ts = pd.DataFrame(
@@ -77,6 +79,8 @@ def test_map_overlap_names():
     assert res4._name != res._name
 
 
+# TODO(pandas) fix Index.is_all_dates deprecation
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 def test_map_overlap_errors():
     # Non-integer
     with pytest.raises(ValueError):
@@ -141,12 +145,19 @@ rolling_method_args_check_less_precise = [
 @pytest.mark.parametrize("center", [True, False])
 def test_rolling_methods(method, args, window, center, check_less_precise):
     if dd._compat.PANDAS_GT_110:
-        check_less_precise = {}
+        if check_less_precise:
+            check_less_precise = {"atol": 1e-3, "rtol": 1e-3}
+        else:
+            check_less_precise = {}
     else:
         check_less_precise = {"check_less_precise": check_less_precise}
+    if dd._compat.PANDAS_GT_120 and method == "count":
+        min_periods = 0
+    else:
+        min_periods = None
     # DataFrame
-    prolling = df.rolling(window, center=center)
-    drolling = ddf.rolling(window, center=center)
+    prolling = df.rolling(window, center=center, min_periods=min_periods)
+    drolling = ddf.rolling(window, center=center, min_periods=min_periods)
     if method == "apply":
         kwargs = {"raw": False}
     else:
@@ -159,8 +170,8 @@ def test_rolling_methods(method, args, window, center, check_less_precise):
     )
 
     # Series
-    prolling = df.a.rolling(window, center=center)
-    drolling = ddf.a.rolling(window, center=center)
+    prolling = df.a.rolling(window, center=center, min_periods=min_periods)
+    drolling = ddf.a.rolling(window, center=center, min_periods=min_periods)
     assert_eq(
         getattr(prolling, method)(*args, **kwargs),
         getattr(drolling, method)(*args, **kwargs),
@@ -168,15 +179,6 @@ def test_rolling_methods(method, args, window, center, check_less_precise):
     )
 
 
-if PANDAS_VERSION <= "0.25.0":
-    filter_panel_warning = pytest.mark.filterwarnings(
-        "ignore::DeprecationWarning:pandas[.*]"
-    )
-else:
-    filter_panel_warning = lambda f: f
-
-
-@filter_panel_warning
 @pytest.mark.parametrize("window", [1, 2, 4, 5])
 @pytest.mark.parametrize("center", [True, False])
 def test_rolling_cov(window, center):
@@ -248,11 +250,17 @@ def test_rolling_repr():
     assert res == "Rolling [window=4,center=False,axis=0]"
 
 
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.skipif(PANDAS_GT_130, reason="win_type changed in pandas master")
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 def test_time_rolling_repr():
     res = repr(dts.rolling("4s"))
     assert res == "Rolling [window=4000000000,center=False,win_type=freq,axis=0]"
 
 
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.skipif(PANDAS_GT_130, reason="win_type changed in pandas master")
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 def test_time_rolling_constructor():
     result = dts.rolling("4s")
     assert result.window == "4s"
@@ -264,13 +272,19 @@ def test_time_rolling_constructor():
     assert result._min_periods == 1
 
 
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.filterwarnings("ignore:win_type:FutureWarning")
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 @pytest.mark.parametrize(
     "method,args,check_less_precise", rolling_method_args_check_less_precise
 )
 @pytest.mark.parametrize("window", ["1S", "2S", "3S", pd.offsets.Second(5)])
 def test_time_rolling_methods(method, args, window, check_less_precise):
     if dd._compat.PANDAS_GT_110:
-        check_less_precise = {}
+        if check_less_precise:
+            check_less_precise = {"atol": 1e-3, "rtol": 1e-3}
+        else:
+            check_less_precise = {}
     else:
         check_less_precise = {"check_less_precise": check_less_precise}
 
@@ -297,7 +311,9 @@ def test_time_rolling_methods(method, args, window, check_less_precise):
     )
 
 
-@filter_panel_warning
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.filterwarnings("ignore:win_type:FutureWarning")
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 @pytest.mark.parametrize("window", ["1S", "2S", "3S", pd.offsets.Second(5)])
 def test_time_rolling_cov(window):
     # DataFrame
@@ -311,6 +327,9 @@ def test_time_rolling_cov(window):
     assert_eq(prolling.cov(), drolling.cov())
 
 
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.filterwarnings("ignore:win_type:FutureWarning")
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 @pytest.mark.parametrize(
     "window,N",
     [("1s", 10), ("2s", 10), ("10s", 10), ("10h", 10), ("10s", 100), ("10h", 100)],
@@ -329,6 +348,9 @@ def test_time_rolling_large_window_fixed_chunks(window, N):
     assert_eq(ddf.rolling(window).mean(), df.rolling(window).mean())
 
 
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.filterwarnings("ignore:win_type:FutureWarning")
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 @pytest.mark.parametrize("window", ["2s", "5s", "20s", "10h"])
 def test_time_rolling_large_window_variable_chunks(window):
     df = pd.DataFrame(
@@ -346,6 +368,8 @@ def test_time_rolling_large_window_variable_chunks(window):
     assert_eq(ddf.rolling(window).mean(), df.rolling(window).mean())
 
 
+# TODO(pandas) fix rolling to be compatible with pandas 1.3.0
+@pytest.mark.filterwarnings(r"ignore:Index\.is_all_dates:FutureWarning")
 @pytest.mark.parametrize("before, after", [("6s", "6s"), ("2s", "2s"), ("6s", "2s")])
 def test_time_rolling(before, after):
     window = before
@@ -388,7 +412,6 @@ def test_rolling_agg_aggregate():
 
 
 @pytest.mark.skipif(not dd._compat.PANDAS_GT_100, reason="needs pandas>=1.0.0")
-@pytest.mark.xfail(da.numpy_compat._numpy_120, reason="sparse-383")
 def test_rolling_numba_engine():
     numba = pytest.importorskip("numba")
     if not dd._compat.PANDAS_GT_104 and LooseVersion(numba.__version__) >= "0.49":
@@ -407,7 +430,7 @@ def test_rolling_numba_engine():
     )
 
 
-@pytest.mark.skipif(dd._compat.PANDAS_GT_100, reason="Requires pandas<1.0.0")
+@pytest.mark.skipif(dd._compat.PANDAS_GT_100, reason="Requires pandas>1.0.0")
 def test_rolling_apply_numba_raises():
     df = pd.DataFrame({"A": range(5), "B": range(0, 10, 2)})
     ddf = dd.from_pandas(df, npartitions=3)
